from tqdm import tqdm
from looker_sdk import models40 as models
from lmanage.configurator.create_object import CreateObject
from lmanage.utils import logger_creation as log_color
# logger = log_color.init_logger(__name__, logger_level)


class CreateBoards(CreateObject):
    def __init__(self, sdk, board_metadata, dashboard_mapping, look_mapping) -> None:
        self.sdk = sdk
        self.dashboard_mapping = dashboard_mapping
        self.look_mapping = look_mapping
        self.board_metadata = board_metadata

    def create_boards(self, bmeta: dict):
        board_body = models.WriteBoard(
            description=bmeta.get('description'),
            title=bmeta.get('title'),
            section_order=bmeta.get('section_order')
        )

        board = self.sdk.create_board(body=board_body)

        return board

    def create_board_section(self, section_object, board_id):
        new_board_section = models.WriteBoardSection(
            board_id=board_id,
            description=section_object.description,
            item_order=section_object.item_order,
            title=section_object.title
        )
        resp = self.sdk.create_board_section(body=new_board_section)
        return resp.id

    def match_dashboard_id(self, source_dashboard_id: str) -> str:
        pass

    def match_look_id(self, source_look_id: str) -> str:
        pass

    def create_board_item(self, target_board_section_id):

        dashboard_id = None
        look_id = None

        if target_board_section_id.dashboard_id:
            dashboard_id = self.match_dashboard_id(
                target_board_section_id.dashboard_id)
        if target_board_section_id.look_id:
            look_id = self.match_look_id(target_board_section_id.look_id)

        new_board_item = models.WriteBoardItem()
        new_board_item.__dict__.update(source_board_item_object.__dict__)
        new_board_item.dashboard_id = dashboard_id
        new_board_item.look_id = look_id
        new_board_item.homepage_section_id = target_board_section_id

        logger.info(
            "Creating item",
            extra={
                "section_id": new_board_item.homepage_section_id,
                "dashboard_id": new_board_item.dashboard_id,
                "look_id": new_board_item.look_id,
                "url": new_board_item.url
            }
        )
        resp = target_sdk.create_homepage_item(new_board_item)
        logger.info("Item created", extra={"id": resp.id})

        return resp

    def execute(self):
        '''
        create boards
        in boards create sections
        in sections create board items
        check content access provisions
        '''
        for board in self.board_metadata:
            nb = self.create_boards(bmeta=board)

            print(nb)



import logging
from looker_sdk import models40 as models
from tqdm import tqdm
from tenacity import retry, wait_fixed, wait_random, stop_after_attempt
from lmanage.configurator.create_object import CreateObject
from lmanage.utils.helpers import nstr


class CreateLooks(CreateObject):
    def __init__(self, sdk, folder_mapping, content_metadata, logger):
        self.sdk = sdk
        self.folder_mapping = folder_mapping
        self.content_metadata = content_metadata
        self.logger = logger

    def execute(self) -> dict:
        look_mapping = []
        for look in tqdm(self.content_metadata, desc="Look Creation", unit="attributes", colour="#2c8558"):
            query = self.__create_query(look_metadata=look)
            created_look = self.__create_look(query.id, look)
            if 'scheduled_plans' in look and len(look['scheduled_plans']) > 0:
                self.__create_scheduled_plans(
                    look['scheduled_plans'], created_look.id)
            temp = {}
            temp['look_mapping'] = {}
            temp['look_mapping'][look.get('look_id')] = created_look.id
            temp['folder_mapping'] = {}
            temp['folder_mapping'][look.get('legacy_folder_id')] = self.folder_mapping.get(
                look.get('legacy_folder_id'))
            look_mapping.append(temp)
        return look_mapping

    @retry(wait=wait_fixed(3) + wait_random(0, 2), stop=stop_after_attempt(5))
    def __create_query(self, look_metadata: dict) -> int:
        '''create a query from look metadata and return the id'''
        query_body = models.WriteQuery(
            model=look_metadata['query_obj']['model'] if look_metadata['query_obj']['model'] else None,
            view=look_metadata['query_obj']['view'] if look_metadata['query_obj']['view'] else None,
            fields=look_metadata['query_obj']['fields'] if look_metadata['query_obj']['fields'] else None,
            pivots=look_metadata['query_obj']['pivots'] if look_metadata['query_obj']['pivots'] else None,
            fill_fields=look_metadata['query_obj']['fill_fields'] if look_metadata['query_obj']['fill_fields'] else None,
            filters=look_metadata['query_obj']['filters'] if look_metadata['query_obj']['filters'] else None,
            filter_expression=look_metadata['query_obj']['filter_expression'] if look_metadata[
                'query_obj']['filter_expression'] else None,
            sorts=look_metadata['query_obj']['sorts'] if look_metadata['query_obj']['sorts'] else None,
            limit=look_metadata['query_obj']['limit'] if look_metadata['query_obj']['limit'] else None,
            column_limit=look_metadata['query_obj']['column_limit'] if look_metadata['query_obj']['column_limit'] else None,
            total=look_metadata['query_obj']['total'] if look_metadata['query_obj']['total'] else None,
            row_total=look_metadata['query_obj']['row_total'] if look_metadata['query_obj']['row_total'] else None,
            subtotals=look_metadata['query_obj']['subtotals'] if look_metadata['query_obj']['subtotals'] else None,
            vis_config=look_metadata['query_obj']['vis_config'] if look_metadata['query_obj']['vis_config'] else None,
            filter_config=look_metadata['query_obj']['filter_config'] if look_metadata['query_obj']['filter_config'] else None,
            visible_ui_sections=look_metadata['query_obj']['visible_ui_sections'] if look_metadata[
                'query_obj']['visible_ui_sections'] else None,
            dynamic_fields=look_metadata['query_obj']['dynamic_fields'] if look_metadata['query_obj']['dynamic_fields'] else None,
            query_timezone=look_metadata['query_obj']['query_timezone'] if look_metadata['query_obj']['query_timezone'] else None
        )
        response = self.sdk.create_query(body=query_body)
        return response

    def __create_look(self, query_id: int, look: dict) -> dict:
        old_folder_id = look.get('legacy_folder_id')
        new_folder_id = self.folder_mapping.get(old_folder_id)
        look_body = models.WriteLookWithQuery(
            title=look.get('title'),
            description=look['description'],
            query_id=query_id,
            folder_id=new_folder_id)
        return self.sdk.create_look(body=look_body)

    def __create_scheduled_plans(self, scheduled_plans, look_id):
        for schedule in scheduled_plans:
            destinations = []
            for d in schedule['scheduled_plan_destination']:
                destination = models.ScheduledPlanDestination()
                destination.__dict__.update(d)
                destinations.append(destination)
            body = models.WriteScheduledPlan(
                name=schedule['name'],
                run_as_recipient=schedule['run_as_recipient'],
                enabled=schedule['enabled'],
                look_id=look_id,
                scheduled_plan_destination=destinations,
                filters_string=nstr(schedule['filters_string']),
                require_results=schedule['require_results'],
                require_no_results=schedule['require_no_results'],
                require_change=schedule['require_change'],
                send_all_results=schedule['send_all_results'],
                crontab=schedule['crontab'],
                timezone=schedule['timezone'],
                datagroup=schedule['datagroup'],
                query_id=schedule['query_id'],
                include_links=schedule['include_links'],
                pdf_paper_size=schedule['pdf_paper_size'],
                pdf_landscape=schedule['pdf_landscape'],
                embed=schedule['embed'],
                color_theme=schedule['color_theme'],
                long_tables=schedule['long_tables'],
                inline_table_width=schedule['inline_table_width'],
            )
            self.sdk.create_scheduled_plan(body=body)



import logging
from rich.logging import RichHandler
import datetime
import os


def create_timestamp():
    timestamp = datetime.datetime.now()
    unix_ts = timestamp.timestamp()
    trunc_unix_ts = unix_ts - (unix_ts % 60)
    return int(trunc_unix_ts)


def init_logger(dunder_name, testing_mode) -> logging.Logger:
    log_format = (
        '%(asctime)s - '
        '%(name)s - '
        '%(funcName)s - '
        '%(levelname)s - '
        '%(message)s'
    )

    logging.basicConfig(
        format=log_format,
        datefmt=".",
        handlers=[RichHandler()],
    )
    logger = logging.getLogger(dunder_name)

    if testing_mode:
        logger.setLevel(logging.DEBUG)
    else:
        logger.setLevel(logging.INFO)

    unix_ts = create_timestamp()

    # Create logs folder if not exists
    os.makedirs('logs', exist_ok=True)

    # Output full log
    fh = logging.FileHandler(f'logs/lomt_{unix_ts}.log')
    fh.setLevel(logging.DEBUG)
    formatter = logging.Formatter(log_format)
    fh.setFormatter(formatter)
    logger.addHandler(fh)

    return logger



from lmanage.utils import looker_object_constructors as loc, errorhandling as eh, logger_creation as log_color
from tqdm import tqdm
from tenacity import retry, wait_fixed, wait_random, stop_after_attempt

class ExtractUserAttributes():
    def __init__(self,  sdk, logger):
        self.sdk = sdk
        self.logger = logger
        self.user_attribute_metadata = self.existing_user_attributes()
        self.all_group_metadata = self.all_group_metadatas()
    
    @retry(wait=wait_fixed(3) + wait_random(0, 2), stop=stop_after_attempt(5))
    def get_all_user_attributes(self) -> dict:
        response = self.sdk.all_user_attributes()
        return response

    def existing_user_attributes(self) -> dict:
        ex_ua = self.get_all_user_attributes()
        resp = [ua for ua in ex_ua if not ua.is_system]
        return resp

    @retry(wait=wait_fixed(3) + wait_random(0, 2), stop=stop_after_attempt(5))
    def all_group_metadatas(self):
        response=self.sdk.all_groups()
        return response 

    @retry(wait=wait_fixed(3) + wait_random(0, 2), stop=stop_after_attempt(5))
    def all_user_attribute_group_value_metadata(self, uaid: int):
        response = self.sdk.all_user_attribute_group_values(uaid)
        return response


    def create_user_attributes(self):
        group_metadata = {
            group.id: group.name for group in self.all_group_metadata}
        response = []
        for ua in tqdm(self.user_attribute_metadata, desc = "User Attribute Capture", unit=" user att", colour="#2c8558"):
            group_assign = self.all_user_attribute_group_value_metadata(uaid=ua.id)
            team_values = []

            for group in group_assign:
                group_name = group_metadata.get(group.group_id)
                teams = {}
                teams[group_name] = group.value
                team_values.append(teams)
            looker_ua = loc.LookerUserAttribute(
                name=ua.name,
                uatype=ua.type,
                hidden_value=ua.value_is_hidden,
                user_view=ua.user_can_view,
                user_edit=ua.user_can_edit,
                default_value=ua.default_value,
                teams_val=team_values
            )
            response.append(looker_ua)
        return response



from looker_sdk import init31
import logging
import coloredlogs
import mapview.instancedata as idata
import mapview.parse_sql_tables as pst
import mapview.compare_content_to_lookml as me
import mapview.matched_field_analysis as mfa
from lmanage.mapview.utils import parse_lookml as pl

logger = logging.getLogger(__name__)
coloredlogs.install(level='INFO')
logging.getLogger("looker_sdk").setLevel(logging.WARNING)


def main(**kwargs):
    div = '-------------------------------------------------------------------'

    ini_file = kwargs.get("ini_file")
    lookml_file_path = kwargs.get("lookml_file_path")
    file_output = kwargs.get("output_path")
    logger.info(div)
    sdk = init31(config_file=ini_file)

    dashboard_metadata = idata.GetCleanInstanceData(sdk=sdk).execute()

    query_elements = pst.ParseSqlTables(dataextract=dashboard_metadata,
                                        sdk=sdk).get_sql_from_elements()

    lookml_files = pl.get_all_lkml_filepaths(starting_path=lookml_file_path)

    elements = me.match_element_to_lookml(
        instancedata=query_elements, view_file=lookml_files)
    da = mfa.MatchedFieldDataAnalysis(matched_fields=elements)

    da.export_to_excel(file_output)
    logger.info('i have finished')


if __name__ == "__main__":
    IP = ('/usr/local/google/home/hugoselbie/code_sample/py/ini/k8.ini')
    FP = ('/usr/local/google/home/hugoselbie/code_sample/py/mapview/test_lookml_files/the_look')

    main(
        ini_file=IP,
        file_path=FP)



from looker_sdk import error
from looker_sdk.sdk.api40 import models
from tqdm import tqdm
from lmanage.utils import logger_creation as log_color
import typing
# logger = log_color.init_logger(__name__, logger_level)


class CreateInstanceFolders():
    def __init__(self, folder_metadata, sdk, logger):
        self.sdk = sdk
        self.folder_metadata = folder_metadata
        self.logger = logger

    def create_folders(self):
        self.__delete_top_level_folders()
        for folder_tree in tqdm(self.folder_metadata, desc="Unesting Folder Data", unit="unested", colour="#2c8558"):
            metadata_list = []
            metadata_list = self.__walk_folder_structure(
                folder_tree=folder_tree, data_storage=metadata_list)
        self.logger.debug('Retrieved YAML folder files')
        self.logger.debug(f'Older metadata = {metadata_list}')
        return metadata_list

    def create_folder_mapping_dict(self, folder_metadata: list) -> dict:
        '''
        create a folder mapping dict with legacy folder id as it's key and new folder id as the value for use in content migration
        '''
        response = {}
        for folder_obj in folder_metadata:
            lid = folder_obj.get('old_folder_id')
            fid = folder_obj.get('new_folder_id')
            response[lid] = fid

        return response

    def __delete_top_level_folders(self):
        def is_top_level_folder(folder):
            return folder['parent_id'] is not None and int(folder['parent_id']) == 1
        all_folders = self.sdk.all_folders('id, name, parent_id')
        for folder in tqdm(all_folders):
            if is_top_level_folder(folder):
                try:
                    self.logger.debug(
                        f'Deleting folder "{folder["name"]}"')
                    self.sdk.delete_folder(folder_id=folder['id'])
                except error.SDKError as e:
                    self.logger.error(e)

    def __walk_folder_structure(self, folder_tree: dict, data_storage: list, parent_id: typing.Optional[str] = None):
        folder_name = folder_tree.get('name')
        try:
            if parent_id is None:
                folder_id = '1'
            else:
                folder_id = self.__create_folder(folder_name=folder_name,
                                                 parent_id=parent_id).get('id')
            temp = {
                'name': folder_name,
                'old_folder_id': folder_tree.get('id'),
                'new_folder_id': folder_id,
                'content_metadata_id': folder_tree.get('content_metadata_id'),
                'team_view': folder_tree.get('team_view'),
                'team_edit': folder_tree.get('team_edit'),
            }
            self.logger.debug(f'Data structure to be appended = {temp}')
            data_storage.append(temp)
            if isinstance(folder_tree.get('subfolder'), list):
                for subfolder in folder_tree.get('subfolder'):
                    self.__walk_folder_structure(subfolder, data_storage,
                                                 parent_id=folder_id)
        except error.SDKError as e:
            self.logger.warn('error %s', e)
            self.logger.warn(
                'you have a duplicate folder called %s with the same parent, this is against best practice and LManage is ignoring it', folder_name)

        # if isinstance(dict_obj.get('subfolder'), list):
        #     for subfolder in dict_obj.get('subfolder'):
        #         self.walk_folder_structure(subfolder, data_storage,
        #                                    parent_name=new_folder_id)

        return data_storage

    def __create_folder(self, folder_name: str, parent_id: str):
        self.logger.debug(f'Creating folder "{folder_name}"')
        folder = self.sdk.create_folder(
            body=models.CreateFolder(
                name=folder_name,
                parent_id=parent_id
            )
        )
        return folder

    # def sync_folders(self, created_folder: list):
    #     all_folders = self.sdk.all_folders()
    #     created_folder_ids = [fobj.get('folder_id') for fobj in created_folder]
    #     all_folder_ids = []

    #     for folder in all_folders:
    #         if folder.is_personal:
    #             pass
    #         elif folder.parent_id is None:
    #             pass
    #         else:
    #             all_folder_ids.append(folder.id)

    #     for fids in all_folder_ids:
    #         if fids not in created_folder_ids:
    #             try:
    #                 self.sdk.delete_folder(folder_id=fids)
    #                 self.logger.debug(
    #                     'deleting folder %s to sync with yaml config', fids)

    #             except error.SDKError as InheritanceError:
    #                 self.logger.debug('root folder has been deleted so %s',
    #                                   InheritanceError)
    #     return 'your folders are in sync with your yaml file'



import os
import lkml
import glob
import logging
import coloredlogs

logger = logging.getLogger(__name__)
coloredlogs.install(level='INFO')
logging.getLogger("looker_sdk").setLevel(logging.WARNING)


def get_all_lkml_filepaths(starting_path):
    logger.info(starting_path)
    lookml_files = [f for f in glob.glob(
        starting_path + "/**/*" + 'lkml', recursive=True)]
    logger.debug(lookml_files)
    return lookml_files


def parse_lookml(path):
    with open(path, 'r') as lookml_file:
        lookml = lkml.load(lookml_file)
        return lookml


def get_parsed_lookml(lookml_file_paths):
    response = []
    for file in lookml_file_paths:
        parsed_lookml = parse_lookml(file)
        logger.debug(parsed_lookml)
        temp = {}
        temp[file] = parsed_lookml
        response.append(temp)
    return response


'''
    parse a LookML file from lmanage.LookML to JSON
    Authors:
            Carl Anderson (carl.anderson@weightwatchers.com)
'''


class LookML():

    def __init__(self, infilepath):
        '''parse the LookML infilepath, convert to JSON, and then read into JSON object
        Args:
            infilepath (str): path to input LookML file
        Returns:
            JSON object of LookML
        '''
        if not os.path.exists(infilepath):
            raise IOError("Filename does not exist: %s" % infilepath)

        self.infilepath = infilepath
        if infilepath.endswith(".model.lkml"):
            self.filetype = 'model'
        elif infilepath.endswith(".view.lkml"):
            self.filetype = 'view'
        elif infilepath.endswith(".explore.lkml"):
            self.filetype = 'explore'
        elif infilepath.endswith(".lkml"):
            self.filetype = 'generic'

        else:
            raise Exception("Unsupported filename " + infilepath)
        self.base_filename = os.path.basename(infilepath)
        self.base_name = self.base_filename.replace(".model.lkml", "").replace(
            ".explore.lkml", "").replace(".view.lkml", "")

        with open(infilepath, 'r') as file:
            self.json_data = lkml.load(file)

    def views(self):
        """get views (if any) from lmanage.the LookML
        Returns:
            views (list) if any, None otherwise
        """
        if 'views' in self.json_data:
            return self.json_data['views']
        return None

    def has_views(self):
        """does this have one or more views?
        Returns:
            bool, whether this has views
        """
        vs = self.views()
        return (vs and len(vs) > 0)

    def explores(self):
        """get explores (if any) from lmanage.the LookML
        Returns:
            explores (list) if any, None otherwise
        """
        if 'explores' in self.json_data:
            return self.json_data['explores']
        return None

    def has_explores(self):
        """does this have one or more explores?
        Returns:
            bool, whether this has explores
        """
        es = self.explores()
        return (es and len(es) > 0)



import json
import ruamel.yaml as yaml
import logging
from lmanage.utils import logger_creation as log_color


class Yaml:
    def __init__(self, yaml_path):
        self.yaml_path = yaml_path
        with open(self.yaml_path, 'r') as file:
            self.parsed_yaml = yaml.load(file, Loader=yaml.BaseLoader)

    def get_metadata(self, type):
        if type == 'folder_metadata':
            metadata = self.__get_folder_metadata()
        elif type == 'permission_set_metadata':
            metadata = self.__get_permission_metadata()
        elif type == 'model_set_metadata':
            metadata = self.__get_model_set_metadata()
        elif type == 'role_metadata':
            metadata = self.__get_role_metadata()
        elif type == 'user_attribute_metadata':
            metadata = self.__get_user_attribute_metadata()
        elif type == 'look_metadata':
            metadata = self.__get_look_metadata()
        elif type == 'dashboard_metadata':
            metadata = self.__get_dashboard_metadata()
        elif type == 'board_metadata':
            metadata = self.__get_board_metadata()

        if not metadata:
            print('')
            # log f'No {type} specified. Please check your yaml file at {self.yaml_path}.')

        return metadata

    def __get_permission_metadata(self):
        response = []
        for objects in self.parsed_yaml:
            if 'permissions' in list(objects.keys()):
                response.append(objects)
        response = [r for r in response if r.get('name') != 'Admin']
        return response

    def __get_model_set_metadata(self):
        response = []
        for objects in self.parsed_yaml:
            if 'models' in list(objects.keys()):
                response.append(objects)
        return response

    def __get_role_metadata(self):
        response = []
        for objects in self.parsed_yaml:
            if 'permission_set' and 'model_set' in list(objects.keys()):
                response.append(objects)
        response = [r for r in response if r.get('name') != 'Admin']
        return response

    def __get_folder_metadata(self):
        response = []
        for objects in self.parsed_yaml:
            if 'content_metadata_id' in list(objects.keys()):
                response.append(objects)
        return response

    def __get_user_attribute_metadata(self):
        response = []
        for objects in self.parsed_yaml:
            if 'hidden_value' in list(objects.keys()):
                response.append(objects)
        return response

    def __get_look_metadata(self):
        response = []
        for objects in self.parsed_yaml:
            if 'query_obj' in list(objects.keys()):
                response.append(objects)
        return response

    def __get_dashboard_metadata(self):
        response = []
        for objects in self.parsed_yaml:
            if 'lookml' in list(objects.keys()):
                response.append(objects)
        return response

    def __get_board_metadata(self):
        response = []
        for objects in self.parsed_yaml:
            if 'board_sections' in list(objects.keys()):
                response.append(objects)
        return response



'''
get dashboards
find elements on dashboards
get query elements of queries
get model files
get explores
get fields

get folder of lookml, or git repo
match dash field to lookml files
match how many times a lookml file has been referenced
show table definition of matched table
'''
from looker_sdk import init31
import logging
import coloredlogs
import instancedata as idata
import parse_lookml as pl
import mapexplores as me
import matched_field_analysis as mfa

logger = logging.getLogger(__name__)
coloredlogs.install(level='INFO')
logging.getLogger("looker_sdk").setLevel(logging.WARNING)


def main(**kwargs):
    div = '-----------------------------------------------------------------------------'

    ini_file = kwargs.get("ini_file")
    lookml_file_path = kwargs.get("file_path")
    logger.info(div)
    sdk = init31(config_file=ini_file)

    dash_to_dash_elements = idata.get_dashboards(sdk=sdk)
    lookml_files = pl.get_all_lkml_filepaths(starting_path=lookml_file_path)

    parsed_lookml_dict = pl.get_parsed_lookml(lookml_file_paths=lookml_files)

    elements = me.match_element_to_lookml(
        instancedata=dash_to_dash_elements, view_file=lookml_files)
    da = mfa.MatchedFieldDataAnalysis(matched_fields=elements)
    print(da.most_used_object(looker_object='fields'))
    print(da.field_to_dashboard_map(dashboard_id_filter=7))
    da.data_analysis()


if __name__ == "__main__":
    IP = ('/usr/local/google/home/hugoselbie/code_sample/py/ini/k8.ini')
    FP = ('/usr/local/google/home/hugoselbie/code_sample/py/mapview/test_lookml_files/the_look')

    main(
        ini_file=IP,
        file_path=FP)



from lmanage.utils import parse_yaml as py, logger_creation as log_color


class LookerConfigReader:
    def __init__(self, config_dir):
        logger_level = "INFO"  # kwargs.get('verbose')
        logger = log_color.init_logger(__name__, logger_level)
        logger.info(
            "----------------------------------------------------------------------------------------"
        )
        logger.info("parsing yaml file")
        self.settings_yaml = py.Yaml(yaml_path=f"{config_dir}/settings.yaml")
        self.content_yaml = py.Yaml(yaml_path=f"{config_dir}/content.yaml")
        # self.logger = log_color.init_logger(__name__, logger_level)

    def read(self):
        self.config = {
            "folder_metadata": self.settings_yaml.get_metadata("folder_metadata"),
            "permission_set_metadata": self.settings_yaml.get_metadata(
                "permission_set_metadata"
            ),
            "model_set_metadata": self.settings_yaml.get_metadata("model_set_metadata"),
            "role_metadata": self.settings_yaml.get_metadata("role_metadata"),
            "user_attribute_metadata": self.settings_yaml.get_metadata(
                "user_attribute_metadata"
            ),
            "look_metadata": self.content_yaml.get_metadata("look_metadata"),
            "dashboard_metadata": self.content_yaml.get_metadata("dashboard_metadata"),
            #     'board_metadata': self.content_yaml.get_metadata('board_metadata'),
        }

    def get_summary(self):
        config = self.config
        look_scheduled_plan_count = sum(
            [
                len(l.get("scheduled_plans")) if l.get("scheduled_plans") else 0
                for l in config["look_metadata"]
            ]
        )
        scheduled_plan_and_alert_counts = [
            (
                len(d.get("scheduled_plans")) if d.get("scheduled_plans") else 0,
                len(d.get("alerts")) if d.get("alerts") else 0,
            )
            for d in config["dashboard_metadata"]
        ]
        scheduled_plan_count = 0
        alert_count = 0

        for first, second in scheduled_plan_and_alert_counts:
            scheduled_plan_count += first
            alert_count += second

        return f"""
        {len(config['permission_set_metadata']):<5} permissions
        {len(config['model_set_metadata']):<5} models
        {len(config['role_metadata']):<5} roles
        {len(config['user_attribute_metadata']):<5} user attributes
        {len(config['look_metadata']):<5} looks
        {look_scheduled_plan_count:<5} look scheduled plans
        {len(config['dashboard_metadata']):<5} dashboards
        {scheduled_plan_count:<5} dashboard scheduled plans
        {alert_count:<5} dashboard alerts
        """



from lmanage.capturator.folder_capturation import folder_config as fc
from lmanage.capturator.user_group_capturation import role_config as rc
from lmanage.capturator.user_attribute_capturation import capture_ua_permissions as cup
from lmanage.capturator.content_capturation import dashboard_capture as dc, look_capture as lc, board_capture as bc
from lmanage.looker_config import LookerConfig
from lmanage.logger_config import setup_logger
from lmanage.looker_auth import LookerAuth

logger = setup_logger()


class LookerApiReader():
    shared_folder_id = '1'

    def __init__(self, ini_file):
        self.sdk = LookerAuth().authenticate(ini_file)

    def get_config(self) -> LookerConfig:
        settings = self.__read_settings()
        content = self.__read_content()
        return LookerConfig(settings, content)

    def __read_settings(self):
        # Folder configuration
        folder_returns = fc.CaptureFolderConfig(
            sdk=self.sdk, logger=logger).execute()
        folder_structure_list = folder_returns[1]
        self.content_folders = [self.shared_folder_id] + folder_returns[0]

        # Roles, Permission Sets & Model Sets
        role_info = rc.ExtractRoleInfo(sdk=self.sdk, logger=logger)
        permission_sets = role_info.extract_permission_sets()
        model_sets = role_info.extract_model_sets()
        roles = role_info.extract_role_info()

        # User Attributes
        user_attributes = cup.ExtractUserAttributes(
            sdk=self.sdk, logger=logger).create_user_attributes()

        return {
            'folder_permissions': folder_structure_list,
            'permission_sets': permission_sets,
            'model_sets': model_sets,
            'roles': roles,
            'user_attributes': user_attributes
        }

    def __read_content(self):
        # Boards
        # boards = bc.CaptureBoards(sdk=sdk).execute()

        # Looks
        looks = lc.LookCapture(
            sdk=self.sdk, content_folders=self.content_folders, logger=logger).execute()

        # Dashboards
        dashboards = dc.CaptureDashboards(
            sdk=self.sdk, content_folders=self.content_folders, logger=logger, all_alerts=self.sdk.search_alerts()).execute()

        return {
            'looks': looks,
            'dashboards': dashboards
        }



from tqdm import tqdm
from looker_sdk import models40 as models, error
from lmanage.utils import logger_creation as log_color
from lmanage.configurator.create_object import CreateObject
from tenacity import retry, wait_fixed, wait_random, stop_after_attempt

# logger = log_color.init_logger(__name__, logger_level)


class CreateDashboards(CreateObject):
    def __init__(self, sdk, folder_mapping, content_metadata, logger) -> None:
        self.sdk = sdk
        self.alert_owner_id = sdk.me().id
        self.folder_mapping = folder_mapping
        self.content_metadata = content_metadata
        self.logger = logger

    def execute(self):
        mapping = self.__create_dashboards()
        self.logger.debug(mapping)
        return mapping
    
    @retry(wait=wait_fixed(3) + wait_random(0, 2), stop=stop_after_attempt(5))
    def __create_dashboard(self, folder_id, lookml) -> None:
        created_dashboard = self.sdk.import_dashboard_from_lookml(
            body=models.WriteDashboardLookml(
                folder_id=folder_id,
                lookml=lookml
            ))
        return created_dashboard

    def __create_dashboards(self) -> None:
        for dashboard in tqdm(self.content_metadata, desc="Dashboard Upload", unit="dashboards", colour="#2c8558"):
            self.logger.debug(type(dashboard))
            old_folder_id = dashboard.get('legacy_folder_id').get('folder_id')
            new_folder_id = self.folder_mapping.get(old_folder_id)
            try:
                created_dashboard = self.__create_dashboard(new_folder_id, dashboard.get('lookml'))
            except error.SDKError as e:
                print(e.message)
                continue

            if 'scheduled_plans' in dashboard and len(dashboard['scheduled_plans']) > 0:
                self.__create_scheduled_plans(
                    dashboard['scheduled_plans'], created_dashboard.id)

            if 'alerts' in dashboard and len(dashboard['alerts']) > 0:
                self.__create_alerts(
                    dashboard['alerts'], dashboard['dashboard_element_alert_counts'], created_dashboard.dashboard_elements)

    @retry(wait=wait_fixed(3) + wait_random(0, 2), stop=stop_after_attempt(5))
    def __create_alerts(self, alerts, alert_counts, dashboard_elements):
        alert_index = 0
        alert_count_index = 0
        for element in dashboard_elements:
            alert_count = int(alert_counts[alert_index])
            for _ in range(alert_count):
                alert = alerts[alert_index]
                field = alert['field']
                element_id = element.id
                self.sdk.create_alert(body=models.WriteAlert(
                    cron=alert['cron'],
                    custom_title=alert['custom_title'],
                    dashboard_element_id=element_id,
                    applied_dashboard_filters=alert['applied_dashboard_filters'],
                    comparison_type=alert['comparison_type'],
                    destinations=alert['destinations'],
                    field={
                        'title': field['title'],
                        'name': field['name']
                    },
                    is_disabled=alert['is_disabled'],
                    is_public=alert['is_public'],
                    threshold=alert['threshold'],
                    owner_id=self.alert_owner_id
                ))
                alert_index += 1
            alert_count_index += 1

    @retry(wait=wait_fixed(3) + wait_random(0, 2), stop=stop_after_attempt(5))
    def __create_scheduled_plans(self, scheduled_plans, dashboard_id):
        for schedule in scheduled_plans:
            destinations = []
            for d in schedule['scheduled_plan_destination']:
                destination = models.ScheduledPlanDestination()
                destination.__dict__.update(d)
                destinations.append(destination)
            body = models.WriteScheduledPlan(
                name=schedule['name'],
                # user_id="1",
                run_as_recipient=schedule['run_as_recipient'],
                enabled=schedule['enabled'],
                dashboard_id=dashboard_id,
                # lookml_dashboard_id=schedule['lookml_dashboard_id'],
                scheduled_plan_destination=destinations,
                filters_string=schedule['filters_string'],
                require_results=schedule['require_results'],
                require_no_results=schedule['require_no_results'],
                require_change=schedule['require_change'],
                send_all_results=schedule['send_all_results'],
                crontab=schedule['crontab'],
                timezone=schedule['timezone'],
                datagroup=schedule['datagroup'],
                query_id=schedule['query_id'],
                include_links=schedule['include_links'],
                pdf_paper_size=schedule['pdf_paper_size'],
                pdf_landscape=schedule['pdf_landscape'],
                embed=schedule['embed'],
                color_theme=schedule['color_theme'],
                long_tables=schedule['long_tables'],
                inline_table_width=schedule['inline_table_width'],
            )
            self.sdk.create_scheduled_plan(body=body)



from tqdm import tqdm
import logging
from lmanage.utils.errorhandling import return_sleep_message
from lmanage.utils.looker_object_constructors import BoardObject
from lmanage.utils import logger_creation as log_color
from yaspin import yaspin
#logger = log_color.init_logger(__name__, logger_level)


class CaptureBoards():
    def __init__(self, sdk):
        self.sdk = sdk

    def handle_board_structure(self, board_response: dict) -> dict:
        board_section_list = board_response.board_sections
        bs_list = []

        for board_section in board_section_list:
            board_item_list = board_section.board_items
            new_bi_list = [board_item.__dict__ for board_item in board_item_list] 
            board_section['board_items'] = new_bi_list
            bs_dict = board_section.__dict__
            bs_list.append(bs_dict)

        board_response['board_sections'] = bs_list

        return board_response 
               

    def get_all_boards(self) -> dict:
        response = []
        with yaspin().white.bold.shark.on_blue as sp:
            sp.text="getting all system board metadata (can take a while)"
            all_system_boards = self.sdk.all_boards()
        
        for board in all_system_boards:
            bresponse = self.handle_board_structure(board_response=board)
            b = BoardObject(
                content_metadata_id=board.content_metadata_id,
                section_order=board.section_order,
                title=board.title,
                primary_homepage=board.primary_homepage,
                description=board.description,
                board_sections=bresponse.board_sections
            )
            response.append(b)
        return response


    def execute(self):
        all_boards = self.get_all_boards()
        return all_boards


import looker_sdk
import ruamel.yaml
import os
from lmanage.looker_config import LookerConfig
from lmanage.logger_config import setup_logger
from lmanage.utils import looker_object_constructors as loc, errorhandling as eh

logger = setup_logger()


class LookerConfigSaver():
    def __init__(self, config_dir):
        self.__init_yaml()
        self.config_dir = config_dir
        if not os.path.exists(config_dir):
            os.makedirs(config_dir)

    def save(self, config: LookerConfig):
        self.__save_settings(config.settings)
        self.__save_content(config.content)

    def __init_yaml(self):
        self.yaml = ruamel.yaml.YAML()
        self.yaml.register_class(loc.LookerFolder)
        self.yaml.register_class(loc.LookerModelSet)
        self.yaml.register_class(loc.LookerPermissionSet)
        self.yaml.register_class(loc.LookerRoles)
        self.yaml.register_class(loc.LookerUserAttribute)
        self.yaml.register_class(loc.LookObject)
        self.yaml.register_class(loc.DashboardObject)
        self.yaml.register_class(loc.AlertObject)
        self.yaml.register_class(loc.AlertAppliedDashboardFilterObject)
        self.yaml.register_class(loc.AlertDestinationObject)
        self.yaml.register_class(loc.AlertFieldObject)
        self.yaml.register_class(loc.AlertFieldFilterObject)
        self.yaml.register_class(loc.BoardObject)
        self.yaml.register_class(looker_sdk.sdk.api40.models.ScheduledPlan)
        self.yaml.register_class(
            looker_sdk.sdk.api40.models.ScheduledPlanDestination)
        self.yaml.register_class(looker_sdk.sdk.api40.models.UserPublic)

    def __save_settings(self, settings):
        # Folder Permissions
        with open(self.__get_settings_path(), 'w') as file:
            fd_yml_txt = '''# FOLDER_PERMISSIONS\n# Opening Session Welcome to the Capturator, this is the Folder place\n# -----------------------------------------------------\n\n'''
            file.write(fd_yml_txt)
            self.yaml.dump(settings['folder_permissions'], file)

        # Roles, Permission Sets & Model Sets
        with open(self.__get_settings_path(), 'a') as file:
            r_yml_txt = '''# Looker Role\n# Opening Session Welcome to the Capturator, this is the Role place\n# -----------------------------------------------------\n\n'''
            file.write(r_yml_txt)
            file.write('\n\n# PERMISSION SETS\n')
            self.yaml.dump(settings['permission_sets'], file)
            file.write('\n\n# MODEL SETS\n')
            self.yaml.dump(settings['model_sets'], file)
            file.write('\n\n# LOOKER ROLES\n')
            self.yaml.dump(settings['roles'], file)

        # User Attributes
        with open(self.__get_settings_path(), 'a') as file:
            file.write('\n\n# USER_ATTRIBUTES\n')
            self.yaml.dump(settings['user_attributes'], file)

    def __save_content(self, content):
        # Boards
        # with open(self.__get_content_path(), 'w+') as file:
        #     file.write('\n\n# BoardData\n')
        #     self.yaml.dump(self.content.boards, file)

        # Looks
        looks = content['looks']
        with open(self.__get_content_path(), 'wb') as file:
            file.write(bytes('\n\n# LookData\n', 'utf-8'))
            if eh.test_object_data(looks):
                self.yaml.dump(looks, file)
            else:
                file.write(bytes('#No Captured Looks', 'utf-8'))

        # Dashboards
        dashboards = content['dashboards']
        with open(self.__get_content_path(), 'ab') as file:
            file.write(bytes('\n\n# Dashboard Content\n', 'utf-8'))
            if eh.test_object_data(dashboards):
                self.yaml.dump(dashboards, file)
            else:
                file.write(bytes('#No Captured Dashboards', 'utf-8'))

    def __get_settings_path(self):
        return f'{self.config_dir}/settings.yaml'

    def __get_content_path(self):
        return f'{self.config_dir}/content.yaml'



#!/usr/bin/python
#
# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import click
import configparser
import os
from functools import wraps
from lmanage.capturator.looker_api_reader import LookerApiReader
from lmanage.capturator.looker_config_saver import LookerConfigSaver
from lmanage.configurator.looker_config_reader import LookerConfigReader
from lmanage.configurator.looker_provisioner import LookerProvisioner
from lmanage.utils import logger_creation as log_color

logger = log_color.init_logger(__name__, testing_mode=False)


@click.group()
@click.version_option()
def lmanage():
    pass


def log_args(**kwargs):
    log_level = kwargs.get('verbose', False)
    logger = log_color.init_logger(__name__, testing_mode=log_level)
    for k, v in kwargs.items():
        if {v} != None:
            logger.info(
                f'You have set {v} for your {k} variable')
        else:
            logger.debug(
                f'There is no value set for {k} please use the `--help` flag to see input parameters.')


def clean_args(**kwargs):
    kwargs['config_dir'] = kwargs['config_dir'].rstrip('/')
    return kwargs


def common_options(f):
    @click.option('-i', '--ini-file',
                  help='Specify API Credentials in an ini file, if no path is given program will assume these values are set as environmental variables as denoted at https://github.com/looker-open-source/sdk-codegen#environment-variable-configuration.')
    @click.option('-cd', '--config-dir',
                  help='Where to save the YAML files to use for instance configuration.', required=True)
    @click.option('-v', '--verbose',
                  is_flag=True,
                  help='Add this flag to get a more verbose version of the returned stout text.')
    @click.option('-f', '--force',
                  is_flag=True,
                  help='Add this flag to skip confirmation prompt for capturing and configuring instance.')
    @click.pass_context
    @wraps(f)
    def decorated_function(ctx, *args, **kwargs):
        log_args(**kwargs)
        kwargs = clean_args(**kwargs)
        return ctx.invoke(f, *args, **kwargs)
    return decorated_function


@lmanage.command()
@common_options
def capturator(**kwargs):
    """Captures security settings for your looker instance"""
    ini_file, config_dir, verbose, force = kwargs.get('ini_file'), kwargs.get(
        'config_dir'), kwargs.get('verbose'), kwargs.get('force')

    try:
        target_url = get_target_url(ini_file)
        if force or (not force and click.confirm(f'\nYou are about to capture settings and content from instance {target_url}. Proceed?')):
            config = LookerApiReader(ini_file).get_config()
            LookerConfigSaver(config_dir).save(config)
    except RuntimeError as e:
        print(e)


@lmanage.command()
@common_options
def configurator(**kwargs):
    """Configures security settings for your looker instance"""
    ini_file, config_dir, verbose, force = kwargs.get('ini_file'), kwargs.get(
        'config_dir'), kwargs.get('verbose'), kwargs.get('force')

    try:
        target_url = get_target_url(ini_file)
        config_reader = LookerConfigReader(config_dir)
        config_reader.read()
        summary = config_reader.get_summary()
        if force or (not force and click.confirm(f'\nYou are about configure instance {target_url} with:\n{summary}\nAre you sure you want to proceed?')):
            LookerProvisioner(ini_file).provision(config_reader.config)
    except RuntimeError as e:
        print(e)


def get_target_url(ini_file):
    if ini_file:
        config_parser = configparser.ConfigParser()
        successful_read = config_parser.read(ini_file)

        if not successful_read:
            raise FileNotFoundError(
                f'Configuration file "{ini_file}" not found.')

        sections = config_parser.sections()
        return config_parser.get(sections[0], 'base_url')
    else:
        target_url = os.getenv('LOOKERSDK_BASE_URL')
        if target_url is None:
            raise ValueError(
                'LOOKERSDK_BASE_URL environment variable not set.')


if __name__ == '__main__':
    lmanage()



from tqdm import tqdm
from lmanage.configurator.create_object import CreateObject


class CleanInstanceContent(CreateObject):
    def __init__(self, sdk, logger) -> None:
        self.sdk = sdk
        self.logger = logger

    def execute(self):
        # self.__delete_boards()
        self.__delete_dashboards()
        self.__delete_looks()
        self.__delete_scheduled_plans()
        self.__empty_dashboard_trash()
        self.__empty_look_trash()

    def __delete_boards(self) -> None:
        all_boards = self.sdk.all_boards()
        delete_board_list = [board.id for board in all_boards]
        for board_id in tqdm(delete_board_list, 'Scrubbing Instance Boards', unit='boards', colour="#2c8558"):
            self.sdk.delete_board(board_id=board_id)
            self.logger.debug(
                f'cleaning the trash board {board_id} from instance')

    def __delete_dashboards(self) -> None:
        all_dashboards = self.sdk.all_dashboards()
        delete_id_list = [
            dash.id for dash in all_dashboards if '::' not in dash.id]
        for dash_id in tqdm(delete_id_list, 'Scrubbing Instance Dashboards', unit='dashboards', colour="#2c8558"):
            self.sdk.delete_dashboard(dashboard_id=dash_id)
            self.logger.debug(
                f'cleaning the trash dashboard {dash_id} from instance')

    def __delete_looks(self) -> None:
        all_looks = self.sdk.all_looks()
        delete_id_list = [look.id for look in all_looks]
        for look_id in tqdm(delete_id_list, 'Scrubbing Instance Looks', unit='looks', colour="#2c8558"):
            self.logger.debug(
                f'Deleting look {look_id} from instance')
            self.sdk.delete_look(look_id=look_id)

    def __delete_scheduled_plans(self) -> None:
        all_schedules = self.sdk.all_scheduled_plans(all_users=True)
        schedule_plan_id_list = [schedule.id for schedule in all_schedules]
        for schedule_plan_id in tqdm(schedule_plan_id_list, 'Scrubbing Instance Schedules', unit='schedules', colour="#2c8558"):
            self.sdk.delete_scheduled_plan(scheduled_plan_id=schedule_plan_id)
            self.logger.debug(
                f'cleaning the trash schedule {schedule_plan_id}')

    def __empty_dashboard_trash(self) -> None:
        trash_dash = self.sdk.search_dashboards(deleted=True)
        trash_dash_id_list = [dash.id for dash in trash_dash]
        for dash_id in tqdm(trash_dash_id_list, 'Emptying Dashboard Trash', unit='dashboards', colour="#2c8558"):
            self.sdk.delete_dashboard(dashboard_id=dash_id)
            self.logger.debug(
                f'cleaning the trash dashboard {dash_id} from instance')

    def __empty_look_trash(self) -> None:
        trash_look = self.sdk.search_looks(deleted=True)
        trash_look_id_list = [look.id for look in trash_look]
        for look_id in tqdm(trash_look_id_list, 'Emptying Look Trash', unit='Look', colour="#2c8558"):
            self.sdk.delete_look(look_id=look_id)
            self.logger.debug(
                f'cleaning the trash dashboard {look_id} from instance')



import logging
import coloredlogs
from lmanage.mapview.utils import parsing_sql
from looker_sdk import error


logger = logging.getLogger(__name__)
coloredlogs.install(level='DEBUG')
logging.getLogger("looker_sdk").setLevel(logging.WARNING)


class ParseSqlTables():
    def __init__(self, dataextract, sdk):
        self.dataextract = dataextract
        self.sdk = sdk

    def parse_sql(self, qid: int):
        """
        Idenfies the base tables and joins used by a Looker query.

        Iterates over a list of PyLookML files identifies view files aand returns a
        list of the sql_table_names if they exist in an object of type 'View'
            qid: (int) query_id from lmanage.a  Looker query
        Returns:
            A list of all the tables that are found from lmanage.a Looker generated
            SQL query.
            For example:
            ['public.order_items','public.inventory_items','public.events']
        Exception:
            If a query is broken for whatever reason the Exception is caught to
            continue the program running
        """
        try:
            sql_response = self.sdk.run_query(
                query_id=qid, result_format="sql")
            if isinstance(sql_response, str):
                tables = parsing_sql.extract_tables(sql_response)
                return tables
            else:
                return sql_response
        except error.SDKError:
            return('No Content')

    def get_sql_from_elements(self):
        """Amends returned SDK System__Activity reponse with sql tables used
        from lmanage.the `parse_sql` function.

        Iterates over the response from lmanage.get_dashboards and runs the parse_sql
        function for each returned dashboard element, returns the list of tables
        and amends the dict response and returns it
        Args:
            sdk: Looker SDK object
            content_results: (dict) response from lmanage.get_dashboards function call
        Returns:
            An amended dict response with the sql columns used by each element
            extracted our of the Looker generated SQL for each dashboard object.
            For example:
        [{'dashboard.id': 1,
             'dashboard_element.id': 1,
             'dashboard_element.type': 'vis',
             'dashboard_element.result_source': 'Lookless',
             'query.model': 'bq',
             'query.view': 'order_items',
             'query.formatted_fields': '["order_items.count"]',
             'query.id': 59,
             'dashboard.title': 'dash_1',
             'look.id': None,
             'sql_joins': ['`looker-private-demo.ecomm.order_items`']}]
        """
        for dash in self.dataextract:
            query_id = dash['query.id']
            sql_value = self.parse_sql(query_id)

            dash['sql_joins'] = sql_value

        return self.dataextract



from looker_sdk.sdk.api40 import models
import json


class GetCleanInstanceData():
    def __init__(self, sdk):
        self.sdk = sdk

    def get_dashboards(self):
        """Uses the Looker SDK System__Activity model to extract dashboard
        and dashboard_element metadata.
        """
        query_config = models.WriteQuery(
            model="system__activity",
            view="dashboard",
            fields=[
                "dashboard.id",
                "dashboard_element.id",
                "dashboard_element.type",
                "dashboard_element.result_source",
                "query.formatted_pivots",
                "query.model",
                "query.view",
                "query.formatted_fields",
                "query.id",
                "dashboard.title",
                "look.id"
            ],
            filters={
                "dashboard_element.type": "-text",
                "dashboard.deleted_date": "NULL"
            },
            limit='5000'
        )
        query_response = self.sdk.run_inline_query(
            result_format='json',
            body=query_config
        )

        query_response = json.loads(query_response)

        return query_response

    def unpivot_query(self, query_id):
        query_metadata = self.sdk.query(query_id=query_id)

        query_body = models.WriteQuery(
            model=query_metadata.model,
            view=query_metadata.view,
            fields=query_metadata.fields,
            filters=query_metadata.filters
        )

        new_query = self.sdk.create_query(body=query_body)
        return new_query.id

    def execute(self):
        instance_data = self.get_dashboards()
        clean_data = []
        for db_element in instance_data:
            if db_element['query.formatted_pivots'] is None:
                clean_data.append(db_element)
            else:
                query_id = db_element['query.id']
                nqid = self.unpivot_query(query_id)
                db_element['query.id'] = nqid
                clean_data.append(db_element)
        return clean_data



import looker_sdk
from lmanage.logger_config import setup_logger
from looker_sdk import error

logger = setup_logger()


class LookerAuth():
    def authenticate(self, ini_file):
        sdk = looker_sdk.init40(
            config_file=ini_file) if ini_file else looker_sdk.init40()
        if self.__auth_check(sdk):
            logger.info('User is successfully authenticated to the API')
            return sdk
        else:
            raise Exception(
                'User is not successfully authenticated.  Please verify credentials in .ini file.')

    def __auth_check(self, sdk):
        try:
            sdk.me()
            return True
        except:
            return False



import lkml
import glob
import logging
import coloredlogs
import ast

import parse_lookml

logger = logging.getLogger(__name__)
coloredlogs.install(level='INFO')
logging.getLogger("looker_sdk").setLevel(logging.WARNING)


def format_str_list(list_str):
    new_list = ast.literal_eval(list_str)
    return new_list


def collate_view_files(parsed_lookml):
    response = []
    for instance_lookml in parsed_lookml:
        for path in instance_lookml.keys():
            if 'view.lkml' in path:
                response.append(instance_lookml)
    return response


def extract_field_names_from_lookml(parsed_lookml, data_storage):

    try:
        dimensions = parsed_lookml['dimensions']
    except KeyError:
        dimensions = []
        logger.warning('no dimensions present in view')
    try:
        measures = parsed_lookml['measures']
    except KeyError:
        measures = []
        logger.warning('no measures present in view')

    view_name = parsed_lookml.get('name')
    for dim in dimensions:
        field_name = dim.get('name')
        full_name = f'{view_name}.{field_name}'
        data_storage[full_name] = view_name

    for meas in measures:
        if len(meas) > 0:
            field_name = meas.get('name')
            full_name = f'{view_name}.{field_name}'
            data_storage[full_name] = view_name

    return data_storage


def match_fields_to_lookml(field_list, lookml):
    matched_fields = []
    for field in field_list:
        match = lookml.get(field, 'no match')
        if match != 'no match':
            matched_fields.append(field)

    print(matched_fields)
    return matched_fields


def extract_view_from_field(field_name):
    view_name = field_name.split('.')[0]
    return view_name


def match_element_to_lookml(instancedata, view_file):
    field_lookup = {}
    for lookml in view_file:
        parsed_lookml = parse_lookml.LookML(lookml)
        if parsed_lookml.has_views():
            for v in parsed_lookml.views():
                extract_field_names_from_lookml(
                    parsed_lookml=v, data_storage=field_lookup)

    matches = []
    for dashboard in instancedata:
        fields_used = format_str_list(dashboard.get('query.formatted_fields'))
        views_in_dash = []
        for field in fields_used:
            x = extract_view_from_field(field)
            if x not in views_in_dash:
                views_in_dash.append(x)
        matched_fields = match_fields_to_lookml(field_list=fields_used,
                                                lookml=field_lookup)

        temp = {}
        temp['used_views'] = views_in_dash
        temp['matched_fields'] = matched_fields
        temp['dashboard'] = dashboard.get('dashboard.id')
        matches.append(temp)
    return matches



import logging
import time
from looker_sdk import error
from looker_sdk.sdk.api40 import models
from lmanage.utils import errorhandling
from tqdm import tqdm
from lmanage.utils.errorhandling import return_sleep_message
from lmanage.utils import logger_creation as log_color
from tenacity import retry, wait_fixed, wait_random, stop_after_attempt
from lmanage.configurator.create_object import CreateObject


class CreateAndProvisionInstanceFolders(CreateObject):
    def __init__(self, folders, sdk, logger):
        self.sdk = sdk
        self.instance_folder_metadata = folders
        self.logger = logger

    @retry(wait=wait_fixed(3) + wait_random(0, 2), stop=stop_after_attempt(5))
    def get_content_access_metadata(self, folder: dict) -> list:
        response = []

        folder_list = self.sdk.all_folders()
        folder_cmaid_lookup = {
            folder.name: folder.content_metadata_id for folder in folder_list
        }

        folder_name = folder.get("name")
        cmaid = folder_cmaid_lookup.get(folder_name)
        temp_dict = {}
        temp_dict["name"] = folder_name
        temp_dict["cmi"] = cmaid
        edit_group = folder.get("team_edit")
        view_group = folder.get("team_view")

        perms = []
        if isinstance(edit_group, list):
            for group in edit_group:
                group_dict = {}
                egmetadata = self.search_looker_groups(name=group)
                group_dict["name"] = group
                group_dict["id"] = egmetadata[0].id
                group_dict["permission"] = "edit"
                perms.append(group_dict)
        else:
            group_dict = {}
            group_dict["name"] = "no_name"
            group_dict["id"] = "no_id"
            group_dict["permission"] = "no_permission"
            perms.append(group_dict)

        if isinstance(view_group, list):
            for group in view_group:
                group_dict = {}
                vgmetadata = self.search_looker_groups(name=group)
                group_dict["name"] = group
                group_dict["id"] = vgmetadata[0].id
                group_dict["permission"] = "view"
                perms.append(group_dict)
        else:
            group_dict = {}
            group_dict["name"] = "no_name"
            group_dict["id"] = "no_id"
            group_dict["permission"] = "no_permission"
            perms.append(group_dict)

        temp_dict["group_permissions"] = perms
        response.append(temp_dict)
        return response

    @retry(wait=wait_fixed(3) + wait_random(0, 2), stop=stop_after_attempt(5))
    def create_content_metadata_access(
        self, group_id: int, permission_input: str, content_metadata_id: int
    ) -> dict:

        self.sdk.create_content_metadata_access(
            body=models.ContentMetaGroupUser(
                content_metadata_id=content_metadata_id,
                permission_type=permission_input,
                group_id=group_id,
            )
        )
        folder = self.sdk.content_metadata(content_metadata_id=content_metadata_id).name
        group = self.search_looker_groups(id=group_id)[0].name
        self.logger.debug(
            f"""--> Successfully permissioned group {group} with {permission_input} access, on folder {folder}."""
        )

    def check_existing_access(self, content_metadata_id: int) -> dict:
        # check for existing access to the folder
        response = {
            access.group_id: access
            for access in self.sdk.all_content_metadata_accesses(
                content_metadata_id=content_metadata_id
            )
        }
        return response

    @retry(wait=wait_fixed(3) + wait_random(0, 2), stop=stop_after_attempt(5))
    def check_folder_ancestors(self, group_id: int, cmaid: int):
        """
        This method
        """

        # getting folder id from content metadata number
        folder_id = self.sdk.content_metadata(content_metadata_id=cmaid).folder_id
        # returning the folder ancestors of the folder (i.e. all folders preceding)
        ancestors_list = self.sdk.folder_ancestors(folder_id=folder_id)
        # refining list to eliminate the Shared folder
        ancestors_list = [folder for folder in ancestors_list if folder.id != "1"]

        # looping through folder ancestors checking their access corresponds with yaml access, if not update
        for ancestor in ancestors_list:
            ancestor_cmaid = ancestor.content_metadata_id
            folder_access = self.check_existing_access(
                content_metadata_id=ancestor_cmaid
            )
            # if this condition is met then do nothing
            if group_id not in folder_access:
                if self.check_folder_inheritance(content_metadata_id=ancestor_cmaid):
                    # N.B. when you update folder inheritance you will create a content metadata access
                    # so recheck value to prevent trying to set an already set value
                    self.update_folder_inheritance(
                        cmaid=ancestor_cmaid, inheritance=False
                    )
                    folder_access = self.check_existing_access(
                        content_metadata_id=ancestor_cmaid
                    )
                    if folder_access:
                        pass
                    else:
                        self.create_content_metadata_access(
                            group_id=group_id,
                            permission_input="view",
                            content_metadata_id=ancestor_cmaid,
                        )
                        self.update_folder_inheritance(
                            cmaid=ancestor_cmaid, inheritance=True
                        )
                else:
                    self.create_content_metadata_access(
                        group_id=group_id,
                        permission_input="view",
                        content_metadata_id=ancestor_cmaid,
                    )

    def check_folder_inheritance(self, content_metadata_id: int) -> bool:
        response = self.sdk.content_metadata(content_metadata_id=content_metadata_id)
        r = response.inherits
        return r

    def check_folder_inheritance_change(self, folder_input: dict) -> bool:
        check = []
        perms = folder_input["group_permissions"]
        for permission in perms:
            if permission["permission"] == "view":
                check.append("true")
            elif permission["permission"] == "edit":
                check.append("true")
            else:
                check.append("false")

        if "true" in check:
            return True
        else:
            return False

    @retry(wait=wait_fixed(3) + wait_random(0, 2), stop=stop_after_attempt(5))
    def update_folder_inheritance(self, cmaid: int, inheritance: bool):
        # don't want to inherit access from lmanage.parent folders
        self.sdk.update_content_metadata(
            content_metadata_id=cmaid,
            body=models.WriteContentMeta(inherits=inheritance),
        )

    @retry(wait=wait_fixed(3) + wait_random(0, 2), stop=stop_after_attempt(5))
    def search_looker_groups(self, id=None, name=None) -> list:
        if id is not None:
            return self.sdk.search_groups(id=id)
        elif name is not None:
            return self.sdk.search_groups(name=name)

    def add_content_access(
        self, cm_accesses: dict, cmaid: int, group_permissions: list
    ) -> dict:

        for group in group_permissions:
            group_id = group.get("id")
            permission = group.get("permission")
            self.check_folder_ancestors(group_id=group_id, cmaid=cmaid)
            self.update_folder_inheritance(cmaid=cmaid, inheritance=False)
            group_name = self.search_looker_groups(id=group_id)[0].name
            self.logger.debug(group_name)

            if group_id in cm_accesses.keys():
                current_access = cm_accesses.get(group_id)

                if current_access.permission_type == permission:
                    self.logger.debug(
                        f"""--> Group {group_name} already has access, to folder {folder_name}
                        no changes made."""
                    )

                else:
                    try:
                        self.sdk.update_content_metadata_access(
                            content_metadata_access_id=current_access.id,
                            body=models.ContentMetaGroupUser(
                                content_metadata_id=cmaid,
                                permission_type=permission,
                                group_id=group_id,
                            ),
                        )
                    except error.SDKError as foldererror:
                        self.logger.debug(foldererror)
                    folder_name = self.sdk.content_metadata(
                        content_metadata_id=cmaid
                    ).name
                    group_name = self.search_looker_groups(id=group_id)[0].name

                    self.logger.debug(
                        f"--> Updating group id {group_name} permission type to {permission} on folder {folder_name}."
                    )

            # no existing access
            # create from lmanage.scratch
            elif permission == "no_permission":
                pass
            else:
                self.create_content_metadata_access(
                    content_metadata_id=cmaid,
                    permission_input=permission,
                    group_id=group_id,
                )

    @retry(wait=wait_fixed(3) + wait_random(0, 2), stop=stop_after_attempt(5))
    def provision_folders_with_group_access(self, content_access_metadata_list: list):
        for folder_tree in tqdm(
            content_access_metadata_list,
            desc="Folder Perms - Group Access",
            unit=" folders provisioned",
            colour="#2c8558",
        ):
            for access_item in folder_tree:
                content_metadata_id = access_item["cmi"]

                self.update_folder_inheritance(
                    cmaid=content_metadata_id, inheritance=False
                )
                gp_permissions = access_item.get("group_permissions")
                self.sync_folder_permission(
                    cmaid=content_metadata_id, gp_permissions=gp_permissions
                )

    def remove_content_access(self, cm_accesses: dict):

        # remove all accesses
        for group_id in cm_accesses.keys():
            try:
                delete_cmi = cm_accesses.get(group_id).id
                self.sdk.delete_content_metadata_access(
                    content_metadata_access_id=delete_cmi
                )
            except error.SDKError as InheritanceError:
                self.logger.debug(
                    """You have an inheritance error in your YAML file possibly 
                            around %s, skipping group_id %s""",
                    delete_cmi,
                    group_id,
                )
                self.logger.debug(InheritanceError)

    @retry(wait=wait_fixed(5) + wait_random(0, 2), stop=stop_after_attempt(5))
    def sync_folder_permission(self, cmaid: int, gp_permissions: list):
        # check for existing access to the folder
        content_metadata_accesses = self.check_existing_access(
            content_metadata_id=cmaid
        )

        # check group permissions and add back 1 by 1
        gp_permissions = [
            perms for perms in gp_permissions if perms.get("id") != "no_id"
        ]
        self.logger.debug(
            "syncing folder permissions for content metadata id %s", cmaid
        )

        if not gp_permissions:
            self.update_folder_inheritance(cmaid=cmaid, inheritance=True)
            self.update_folder_inheritance(cmaid=cmaid, inheritance=False)
        else:
            # remove folder content accesses
            self.remove_content_access(cm_accesses=content_metadata_accesses)
            # check for existing access to the folder
            content_metadata_accesses = self.check_existing_access(
                content_metadata_id=cmaid
            )

            self.add_content_access(
                cm_accesses=content_metadata_accesses,
                cmaid=cmaid,
                group_permissions=gp_permissions,
            )

    def remove_all_user_group(self, content_access_metadata_list: list):

        # remove parent Shared group instance access
        try:
            folder = "1"
            self.sdk.update_content_metadata_access(
                content_metadata_access_id=folder,
                body=models.ContentMetaGroupUser(
                    permission_type="view", content_metadata_id=folder, group_id=folder
                ),
            )
        except error.SDKError:
            self.logger.debug("All Users group already configured")
        clean = list()
        for avt in content_access_metadata_list:
            temp = {}
            temp["name"] = avt[0].get("name")
            temp["cmi"] = avt[0].get("cmi")
            clean.append(temp)
        res_list = [i for n, i in enumerate(clean) if i not in clean[n + 1 :]]

        for access_item in res_list:
            time.sleep(1)

            content_metadata_id = access_item["cmi"]

            # check for existing access to the folder
            content_metadata_accesses = {
                access.group_id: access
                for access in self.sdk.all_content_metadata_accesses(
                    content_metadata_id=content_metadata_id
                )
            }

            for id, value in content_metadata_accesses.items():
                self.logger.debug(f"Checking item {value.id}")
                if value.group_id == 1:
                    cmaid = value.id
                    self.sdk.delete_content_metadata_access(
                        content_metadata_access_id=cmaid
                    )

    def execute(self):
        # CONFIGURE FOLDERS WITH EDIT AND VIEW ACCESS
        content_access_metadata = []
        for folder_obj in tqdm(
            self.instance_folder_metadata,
            desc="Folder Perms - Edit/View Access",
            unit=" folders",
            colour="#2c8558",
        ):
            r = self.get_content_access_metadata(folder=folder_obj)
            content_access_metadata.append(r)

        # ADD AND SYNC CONTENT VIEW ACCESS WITH YAML
        self.provision_folders_with_group_access(
            content_access_metadata_list=content_access_metadata
        )
        self.remove_all_user_group(content_access_metadata_list=content_access_metadata)



from lmanage.looker_auth import LookerAuth
from lmanage.configurator.user_attribute_configuration import create_and_assign_user_attributes as cuap
from lmanage.configurator.folder_configuration import create_and_provision_instance_folders as cfp, create_instance_folders as cf
from lmanage.configurator.user_group_configuration import create_instance_groups as gc, create_instance_roles as up, create_role_base as rc
from lmanage.configurator.content_configuration import clean_instance_content as ccp, create_looks as cl, create_dashboards as cd, create_boards as cb
from lmanage.utils import logger_creation as log_color


class LookerProvisioner():
    def __init__(self, ini_file):
        self.sdk = LookerAuth().authenticate(ini_file)
        logger_level = 'INFO'  # kwargs.get('verbose')
        self.logger = log_color.init_logger(__name__, logger_level)
        self.logger.info(
            '----------------------------------------------------------------------------------------')
        self.logger.info('provisioning')

    def provision(self, metadata):
        ###############
        # Role Config #
        ###############
        # Create Permission and Model Sets
        rc.CreateRoleBase(permissions=metadata['permission_set_metadata'],
                          model_sets=metadata['model_set_metadata'], sdk=self.sdk, logger=self.logger).execute()

        #################
        # Folder Config #
        #################
        # CREATE NEW FOLDERS
        folder_objects = cf.CreateInstanceFolders(
            folder_metadata=metadata['folder_metadata'], sdk=self.sdk, logger=self.logger)
        created_folder_metadata = folder_objects.create_folders()

        # CREATE FOLDER TO FOLDER Dict
        folder_mapping_obj = folder_objects.create_folder_mapping_dict(
            folder_metadata=created_folder_metadata)

        ################
        # Group Config #
        ################
        # CREATE NEW GROUPS FROM YAML FILE TEAM VALUES
        gc.CreateInstanceGroups(
            folders=created_folder_metadata,
            user_attributes=metadata['user_attribute_metadata'],
            roles=metadata['role_metadata'],
            sdk=self.sdk,
            logger=self.logger).execute()

        ###########################
        # Folder Provision Config #
        ###########################
        # CREATE NEW GROUPS FROM YAML FILE TEAM VALUES
        cfp.CreateAndProvisionInstanceFolders(
            folders=created_folder_metadata,
            sdk=self.sdk, logger=self.logger).execute()

        ###############
        # Role Config #
        ###############
        up.CreateInstanceRoles(roles=metadata['role_metadata'],
                               sdk=self.sdk,
                               logger=self.logger).execute()

        #########################
        # User Attribute Config #
        #########################
        # FIND UNIQUE USER ATTRIBUTES AND ATTRIBUTE TO TEAM
        cuap.CreateAndAssignUserAttributes(
            user_attributes=metadata['user_attribute_metadata'],
            sdk=self.sdk,
            logger=self.logger).execute()

        ############################
        # Content Transport Config #
        ############################
        # EMPTY TRASH CAN OF ALL DELETED CONTENT
        ccp.CleanInstanceContent(sdk=self.sdk, logger=self.logger).execute()

        # FIND LOOKS AND REMAKE THEM
        cl.CreateLooks(
            sdk=self.sdk,
            folder_mapping=folder_mapping_obj,
            content_metadata=metadata['look_metadata'],
            logger=self.logger).execute()

        # Find DASHBOARDS AND REMAKE THEM
        cd.CreateDashboards(
            sdk=self.sdk,
            folder_mapping=folder_mapping_obj,
            content_metadata=metadata['dashboard_metadata'],
            logger=self.logger).execute()



"""
Copyright 2021 Google LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import os
import logging
from lmanage.pathlib import Path
import json
import pandas as pd

logger = logging.getLogger(__name__)


def create_df(data):
    logger.debug(type(data))
    data = json.loads(data) if isinstance(data, str) else data
    df = pd.DataFrame(data)
    return df


def check_ini(ini):
    cwd = Path.cwd()

    if ini:
        parsed_ini_file = cwd.joinpath(ini)
    else:
        parsed_ini_file = None

    if os.path.isfile(parsed_ini_file):
        logger.info(f'opening your ini file at {parsed_ini_file}')
        return parsed_ini_file
    else:
        logger.debug(
            f'the path to your ini file is not valid at {parsed_ini_file}')



import ruamel.yaml
from lmanage.utils.helpers import xstr

yaml = ruamel.yaml.YAML()


@yaml.register_class
class LookerFolder:
    def __init__(self, id, folder_metadata, access_list):
        self.parent_id = folder_metadata.get("parent_id")
        self.id = id
        self.name = folder_metadata.get("name")
        self.subfolder = []
        self.content_metadata_id = folder_metadata.get("content_metadata_id")
        self.team_edit = self.breakup_access_list(
            access_list=access_list, access_type="edit"
        )
        self.team_view = self.breakup_access_list(
            access_list=access_list, access_type="view"
        )

    def add_child_folder(self, ref):
        self.subfolder.append(ref)

    def breakup_access_list(self, access_list, access_type):
        response = []
        for access in access_list:
            team = access.get(access_type, None)
            if team is not None:
                response.append(team)
        return response


class LookerPermissionSet:
    def __init__(self, permissions, name):
        self.permissions = permissions
        self.name = name


class LookerUserAttribute:
    def __init__(
        self,
        teams_val: dict,
        name: str,
        uatype: bool,
        hidden_value: bool,
        user_view,
        user_edit,
        default_value,
    ) -> object:
        self.name = name
        self.uatype = uatype
        self.hidden_value = hidden_value
        self.user_view = str(user_view)
        self.user_edit = str(user_edit)
        self.default_value = default_value
        self.teams = teams_val


class LookerModelSet:
    def __init__(self, models, name):
        self.models = models
        self.name = name


class LookerRoles:
    def __init__(self, permission_set, model_set, teams, name):
        self.permission_set = permission_set
        self.model_set = model_set
        self.teams = teams
        self.name = name


class LookerGroup:
    def __init__(self, id, group_metadata):
        self.name = group_metadata.name
        self.id = id
        self.children = []


class LookObject:
    def __init__(
        self, description, query_obj, title, legacy_folder_id, look_id, scheduled_plans
    ):
        self.legacy_folder_id = legacy_folder_id
        self.look_id = look_id
        self.title = title
        self.query_obj = query_obj
        self.description = description
        self.scheduled_plans = scheduled_plans


class DashboardObject:
    def __init__(
        self,
        legacy_folder_id,
        lookml,
        dashboard_id,
        dashboard_slug,
        dashboard_element_alert_counts,
        scheduled_plans,
        alerts,
    ) -> None:
        self.legacy_folder_id = legacy_folder_id
        self.lookml = lookml
        self.dashboard_id = dashboard_id
        self.dashboard_slug = dashboard_slug
        self.dashboard_element_alert_counts = dashboard_element_alert_counts
        self.scheduled_plans = scheduled_plans
        self.alerts = alerts


class AlertObject:
    def __init__(self, alert) -> None:
        self.applied_dashboard_filters = [
            AlertAppliedDashboardFilterObject(f)
            for f in alert.get("applied_dashboard_filters")
        ]
        self.comparison_type = alert.get("comparison_type")
        self.cron = alert.get("cron")
        self.custom_title = xstr(alert.get("custom_tile"))
        # self.dashboard_element_id = alert.get('dashboard_element_id')
        self.description = xstr(alert.get("description"))
        self.destinations = [
            AlertDestinationObject(d) for d in alert.get("destinations")
        ]
        self.field = AlertFieldObject(alert.get("field"))
        self.is_disabled = alert.get("is_disabled")
        self.is_public = alert.get("is_public")
        self.disabled_reason = xstr(alert.get("disabled_reason"))
        # self.investigative_content_type = xstr(alert.get(
        #     'investigative_content_type'))
        # self.investigative_content_id = alert.get('investigative_content_id')
        # self.lookml_dashboard_id = alert.get('lookml_dashboard_id')
        # self.lookml_link_id = alert.get('lookml_link_id')
        self.owner_id = alert.get("owner_id")
        self.threshold = alert.get("threshold")
        # self.time_series_condition_state = alert.get(
        #     'time_series_condition_state')


class AlertAppliedDashboardFilterObject:
    def __init__(self, filter):
        self.filter_title = filter.get("title")
        self.field_name = filter.get("title")
        self.filter_value = filter.get("title")
        self.filter_description = xstr(filter.get("title"))


class AlertDestinationObject:
    def __init__(self, destination) -> None:
        self.destination_type = destination.get("destination_type")
        self.email_address = destination.get("email_address")


class AlertFieldObject:
    def __init__(self, alert_field) -> None:
        self.title = alert_field.get("title")
        self.name = alert_field.get("name")
        self.filter = [
            AlertFieldFilterObject(filter) for filter in alert_field.get("filter")
        ]


class AlertFieldFilterObject:
    def __init__(self, alert_field_filter) -> None:
        self.field_name = alert_field_filter.field_name
        self.field_value = alert_field_filter.field_value
        self.filter_value = alert_field_filter.filter_value


class BoardObject:
    def __init__(
        self,
        content_metadata_id,
        section_order,
        title,
        primary_homepage,
        board_sections,
        description,
    ) -> None:
        self.content_metadata_id = content_metadata_id
        self.section_order = section_order
        self.title = title
        self.primary_homepage = primary_homepage
        self.board_sections = board_sections
        self.description = description



from looker_sdk import error
from looker_sdk.sdk.api40 import models
from lmanage.utils.errorhandling import return_error_message
from tqdm import tqdm
from tenacity import retry, wait_fixed, wait_random, stop_after_attempt
from lmanage.configurator.create_object import CreateObject


class CreateInstanceGroups(CreateObject):
    def __init__(self, folders, user_attributes, roles, sdk, logger) -> None:
        self.folder_metadata = folders
        self.user_attribute_metadata = user_attributes
        self.role_metadata = roles
        self.sdk = sdk
        self.logger = logger

    def extract_teams(self, container, data_storage, ua: bool):
        for team in container:
            team_list = team.get('teams')
            if len(team_list) > 0:
                if ua:
                    for team_val in team_list:
                        team_app = list(team_val.keys())[0]
                        data_storage.append(team_app)
                else:
                    for team_val in team_list:
                        data_storage.append(team_val)
        return data_storage

    def extract_folder_teams(self, container, data_storage):
        for folder_element in container:
            if isinstance(folder_element.get('team_edit'), list):
                edit_group = folder_element.get('team_edit')
                for group in edit_group:
                    data_storage.append(group)
            if isinstance(folder_element.get('team_view'), list):
                view_group = folder_element.get('team_view')
                for group in view_group:
                    data_storage.append(group)
            else:
                pass

    @retry(wait=wait_fixed(3) + wait_random(0, 2), stop=stop_after_attempt(5))
    def search_looker_groups(self, sdk, group_name: str) -> list:
        s = sdk.search_groups(name=group_name)
        return s

    def create_group_if_not_exists(self,
                                   sdk,
                                   group_name: str) -> dict:
        """ Create a Looker Group and add Group attributes

        :group_name: Name of a Looker group to create.
        :rtype: Looker Group object.
        """
        # get group if exists
        try:
            self.logger.debug(f'Creating group "{group_name}"')
            group = sdk.create_group(
                body=models.WriteGroup(
                    can_add_to_content_metadata=True,
                    name=group_name
                )
            )
            return group
        except error.SDKError as grouperr:
            err_msg = return_error_message(grouperr)
            self.logger.debug(
                'You have hit a warning creating your group; warning = %s', err_msg)
            self.logger.debug(grouperr)
            group = self.search_looker_groups(
                sdk=self.sdk, group_name=group_name)
            return group[0]

    def get_instance_group_metadata(self,
                                    sdk,
                                    unique_group_list: list) -> list:
        group_metadata = []

        for group_name in tqdm(unique_group_list, desc="Creating User Groups", unit=" user groups", colour="#2c8558"):
            group = self.create_group_if_not_exists(sdk, group_name)
            temp = {}
            temp['group_id'] = group.id
            temp['group_name'] = group.name
            group_metadata.append(temp)

        return group_metadata

    def sync_groups(self,
                    group_name_list: list) -> str:

        all_groups = self.sdk.all_groups()
        group_dict = {group.name: group.id for group in all_groups}
        # Deleting Standard Groups
        try:
            del group_dict['All Users']
        except:
            pass

        for group_name in group_dict.keys():
            if group_name not in group_name_list:
                self.sdk.delete_group(group_id=group_dict[group_name])
                self.logger.info(
                    f'deleting group {group_name} to sync with yaml config')

        return 'your groups are in sync with your yaml file'

    def execute(self):
        team_list = []
        # extracting user attribute teams
        self.extract_teams(container=self.user_attribute_metadata,
                           data_storage=team_list, ua=True)
        # extracting role based teams
        self.extract_teams(
            container=self.role_metadata, data_storage=team_list, ua=False)
        # extract nested folder access teams
        self.extract_folder_teams(
            container=self.folder_metadata, data_storage=team_list)

        team_list = list(set(team_list))

        # create all the groups
        self.get_instance_group_metadata(
            sdk=self.sdk, unique_group_list=team_list)
        self.sync_groups(group_name_list=team_list)



import logging


def setup_logger():
    # Create a custom logger
    logger = logging.getLogger(__name__)

    # Set the level of logging.
    # You might want to use logging.DEBUG or logging.ERROR in other scenarios
    logger.setLevel(logging.INFO)

    # Create handlers. Here, we create a StreamHandler (for stdout) and a FileHandler (for writing logs to a file)
    c_handler = logging.StreamHandler()
    f_handler = logging.FileHandler('file.log')
    c_handler.setLevel(logging.INFO)
    f_handler.setLevel(logging.ERROR)

    # Create formatters and add it to handlers
    c_format = logging.Formatter('%(name)s - %(levelname)s - %(message)s')
    f_format = logging.Formatter(
        '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    c_handler.setFormatter(c_format)
    f_handler.setFormatter(f_format)

    # Add handlers to the logger
    logger.addHandler(c_handler)
    logger.addHandler(f_handler)

    return logger



from lmanage.attr import field
import lkml
import glob
import logging
import coloredlogs
import ast
import pandas as pd
import xlsxwriter

logger = logging.getLogger(__name__)
coloredlogs.install(level='INFO')
logging.getLogger("looker_sdk").setLevel(logging.WARNING)


class MatchedFieldDataAnalysis():
    def __init__(self, matched_fields):
        self.matched_data = matched_fields
        self.df = pd.DataFrame(self.matched_data)

    def create_df(self):
        df = pd.DataFrame(self.matched_data)
        return df

    def most_used_object(self, looker_object):
        if looker_object == 'views':
            xploded = self.df.explode('used_views')
            xploded = xploded.groupby(['used_views'])['dashboard'].count()
            return xploded
        elif looker_object == 'fields':
            xploded = self.df.explode('matched_fields')
            xploded = xploded.groupby(['matched_fields'])['dashboard'].count()
            xploded = xploded.groupby(['used_views'])[
                'dashboard'].count().rename('dashboard_count').sort_values(ascending=False)
            return xploded

    def field_to_dashboard_map(self, field_filter=None, dashboard_id_filter=None):
        xploded = self.df.explode('matched_fields')
        if field_filter:
            xploded = xploded.filter(like=field_filter, axis=0)
        if dashboard_id_filter:
            xploded = xploded.filter(like=dashboard_id_filter)
        return xploded

    def data_analysis(self):
        df = self.create_df()
        print(df)

    def export_to_excel(self, output):
        writer = pd.ExcelWriter(output, engine='xlsxwriter')
        dashboard_match = self.field_to_dashboard_map()
        used_views = self.most_used_object(looker_object='views')
        used_obj = self.most_used_object(looker_object='fields')

        # Write each dataframe to a different worksheet.
        dashboard_match.to_excel(writer, sheet_name='DashboardToLookMLMapping')
        used_obj.to_excel(writer, sheet_name='MostUsedDimensions')
        used_views.to_excel(writer, sheet_name='MostUsedViews')

        # Close the Pandas Excel writer and output the Excel file.
        writer.save()



"""
Copyright 2021 Google LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import itertools
from lmanage.sqlparse import parse
from lmanage.sqlparse.sql import IdentifierList, Identifier
from lmanage.sqlparse.tokens import Keyword, DML


def is_subselect(parsed):
    if not parsed.is_group:
        return False
    for item in parsed.tokens:
        if item.ttype is DML and item.value.upper() == 'SELECT':
            return True
    return False


def extract_from_part(parsed):
    from_seen = False
    for item in parsed.tokens:
        if item.is_group:
            for x in extract_from_part(item):
                yield x
        if from_seen:
            if is_subselect(item):
                for x in extract_from_part(item):
                    yield x
            elif item.ttype is Keyword and item.value.upper() in ['ORDER', 'GROUP', 'BY', 'HAVING', 'GROUP BY']:
                from_seen = False
                StopIteration
            else:
                yield item
        if item.ttype is Keyword and item.value.upper() == 'FROM':
            from_seen = True


def extract_table_identifiers(token_stream):
    for item in token_stream:
        if isinstance(item, IdentifierList):
            for identifier in item.get_identifiers():
                value = identifier.value.replace('"', '').lower()
                yield value
        elif isinstance(item, Identifier):
            value = item.value.replace('"', '').lower()
            yield value


def extract_tables(sql):
    # let's handle multiple statements in one sql string
    extracted_tables = []
    statements = list(parse(sql))
    for statement in statements:
        if statement.get_type() != 'UNKNOWN':
            stream = extract_from_part(statement)
            extracted_tables.append(
                set(list(extract_table_identifiers(stream))))
    rough_tables = list(itertools.chain(*extracted_tables))
    rough_tables = [elem.split(' ') for elem in rough_tables]
    clean_list = []
    for elem in rough_tables:
        select = []
        for i in elem:
            if i:
                select.append(i)

        if len(select) < 4:
            clean_list.append(select)

    clean_table = [elem[0].strip('\n')
                   for elem in clean_list if not elem[0].startswith('z__')]
    # print(clean_table)
    return clean_table


if __name__ == '__main__':
    sql = """    
    -- generate derived table source_query_rank
-- Building temporary derived table system__activity::source_query_rank on instance 7ecf10dbe337872a19ea64ca19132ba5
CREATE TEMPORARY TABLE looker_tmp.source_query_rank SELECT
    history.SOURCE  AS `sorted_source`,
    COUNT(CASE WHEN (history.status NOT LIKE 'cache_only_miss' OR history.status IS NULL) THEN 1 ELSE NULL END) AS `query_run_count`
FROM history
WHERE (((
            CONVERT(history.CREATED_AT USING utf8mb4)
            ) >= ((CURDATE())) AND (
            CONVERT(history.CREATED_AT USING utf8mb4)
            ) < ((DATE_ADD(CURDATE(),INTERVAL 1 day))))) AND (history.SOURCE ) IS NOT NULL
GROUP BY
    1
ORDER BY
    COUNT(CASE WHEN (history.status NOT LIKE 'cache_only_miss' OR history.status IS NULL) THEN 1 ELSE NULL END) DESC
-- finished source_query_rank => looker_tmp.source_query_rank
SELECT
    (DATE_FORMAT(
            CONVERT(history.CREATED_AT USING utf8mb4)
           ,'%Y-%m-%d %H')) AS `history.created_hour`,
    source_query_rank.query_run_count AS `source_query_rank.query_run_count`,
    source_query_rank.sorted_source AS `source_query_rank.sorted_source`,
        (MOD((DAYOFWEEK(
            CONVERT(history.CREATED_AT USING utf8mb4)
           ) - 1) - 1 + 7, 7)) AS `history.created_day_of_week_index`,
        (DATE_FORMAT(
            CONVERT(history.CREATED_AT USING utf8mb4)
           ,'%W')) AS `history.created_day_of_week`,
    COUNT(DISTINCT CASE WHEN (history.status NOT LIKE 'cache_only_miss' OR history.status IS NULL) THEN history.id ELSE NULL END) AS `history.query_run_count`
FROM history
LEFT JOIN looker_tmp.source_query_rank AS source_query_rank ON history.SOURCE = source_query_rank.sorted_source
WHERE (((
            CONVERT(history.CREATED_AT USING utf8mb4)
            ) >= ((CURDATE())) AND (
            CONVERT(history.CREATED_AT USING utf8mb4)
            ) < ((DATE_ADD(CURDATE(),INTERVAL 1 day)))))
GROUP BY
    1,
    2,
    3,
    4,
    5
ORDER BY
    (DATE_FORMAT(
            CONVERT(history.CREATED_AT USING utf8mb4)
           ,'%Y-%m-%d %H')) DESC
LIMIT 5000
    """

    print(extract_tables(sql))



from os import name
from lmanage.utils import logger_creation as log_color

# logger = log_color.init_logger(__name__, logger_level)


class FolderConfig():
    ''' Class to read in folder metadata and unested_folder_data '''

    def __init__(self, folders):
        self.folder_metadata = folders

    def unnest_folder_data(self):
        response = []

        for d in self.folder_metadata:
            folder_dict = d
            metadata_list = []
            metadata_list = self.walk_folder_structure(
                dict_obj=folder_dict,
                data_storage=metadata_list,
                parent_id='1')

            response.append(metadata_list)

        logger.info('retrieved yaml folder files')
        logger.debug('folder metadata = %s', response)

        return response

    def walk_folder_structure(self, dict_obj: dict, data_storage: list, parent_id: str):
        temp = {}
        temp['name'] = dict_obj.get('name')
        temp['legacy_id'] = dict_obj.get('id')
        temp['team_edit'] = dict_obj.get('team_edit')
        temp['team_view'] = dict_obj.get('team_view')
        temp['parent_id'] = parent_id
        logger.debug('data_structure to be appended = %s', temp)
        data_storage.append(temp)

        if isinstance(dict_obj.get('subfolder'), list):
            for subfolder in dict_obj.get('subfolder'):
                self.walk_folder_structure(subfolder, data_storage,
                                           parent_id=dict_obj.get('name'))

        return data_storage



from tqdm import tqdm
import logging
from lmanage.utils import looker_object_constructors as loc, errorhandling as eh, logger_creation as log_color
from tenacity import retry, wait_fixed, wait_random, stop_after_attempt
from yaspin import yaspin


class CaptureDashboards():
    def __init__(self, sdk, content_folders: dict, logger, all_alerts):
        self.sdk = sdk
        self.content_folders = content_folders
        self.logger = logger
        self.all_alerts = all_alerts

    def get_all_dashboards(self) -> dict:
        scrub_dashboards = {}
        with yaspin().white.bold.shark.on_blue as sp:
            sp.text = "getting all system dashboard metadata (can take a while)"
            all_dashboards = self.sdk.all_dashboards(fields="id,folder, slug")

        for dash in all_dashboards:
            self.logger.debug(self.content_folders)
            if dash.folder.id in self.content_folders:
                scrub_dashboards[dash.id] = {}
                scrub_dashboards[dash.id]['folder_id'] = dash.folder.id
                scrub_dashboards[dash.id]['slug'] = dash.slug
        # logger.info(scrub_dashboards)
        return scrub_dashboards

    @retry(wait=wait_fixed(3) + wait_random(0, 2), stop=stop_after_attempt(5))
    def get_looker_dashboard(self, looker_dashboard_id, fields='dashboard_elements'):
        dashboard_elements = self.sdk.dashboard(
            looker_dashboard_id, fields)
        return dashboard_elements

    @retry(wait=wait_fixed(3) + wait_random(0, 2), stop=stop_after_attempt(5))
    def get_looker_schedule_plan_for_dashboard(self, looker_dashboard_id):
        dash_schedule_plans = self.sdk.scheduled_plans_for_dashboard(
            looker_dashboard_id, all_users=True)
        return dash_schedule_plans

    def get_dashboard_lookml(self, all_dashboards: dict) -> list:
        # logging.info("Beginning Dashboard Capture:")
        response = []
        for dashboard_id in tqdm(all_dashboards, desc="Dashboard Capture", unit="dashboards", colour="#2c8558"):
            lookml = None
            trys = 0
            if "::" in dashboard_id:
                continue
            else:
                dashboard_elements = self.get_looker_dashboard(
                    looker_dashboard_id=dashboard_id)['dashboard_elements']
                dashboard_element_ids = [e['id'] for e in dashboard_elements]
                dashboard_element_alert_counts = [
                    e['alert_count'] for e in dashboard_elements]
                scheduled_plans = self.get_looker_schedule_plan_for_dashboard(
                    looker_dashboard_id=dashboard_id)
                alerts = [
                    loc.AlertObject(alert) for alert in self.all_alerts if alert['dashboard_element_id'] in dashboard_element_ids]

                while lookml is None and trys < 5:
                    trys += 1
                    try:
                        lookml = self.sdk.dashboard_lookml(
                            dashboard_id=dashboard_id)
                    except:
                        eh.return_sleep_message(call_number=trys, quiet=True)

                if lookml:
                    self.logger.debug(lookml.lookml)
                    captured_dashboard = loc.DashboardObject(
                        legacy_folder_id=all_dashboards.get(dashboard_id),
                        lookml=lookml.lookml,
                        dashboard_id=dashboard_id,
                        dashboard_slug=all_dashboards.get(
                            dashboard_id).get('slug'),
                        dashboard_element_alert_counts=dashboard_element_alert_counts,
                        scheduled_plans=scheduled_plans,
                        alerts=alerts
                    )
                    response.append(captured_dashboard)
        return response

    def execute(self):
        all_dashboards = self.get_all_dashboards()
        captured_dash = self.get_dashboard_lookml(all_dashboards)
        return captured_dash
